@@ -10,14 +10,22 @@ use Tie::Hash::Indexed;
 use Hash::Merge::Simple qw(merge);
 use Scalar::Util 'blessed';
 use Data::Dumper;
-our $VERSION = '0.070';
+our $VERSION = '0.071';
 $VERSION = eval $VERSION;    # see L<perlmodstyle>
-use HTML::FormFu::Util qw/require_class/;
+
+use HTML::FormFu::ExtJS::Util qw(
+    ext_class_of
+);
+
 
 =head1 NAME
 
 HTML::FormFu::ExtJS - Render and validate ExtJS forms using HTML::FormFu
 
+=head1 VERSION
+
+version 0.071
+
 =head1 DESCRIPTION
 
 This module allows you to render ExtJS forms without changing your HTML::FormFu config file.
@@ -40,7 +48,7 @@ B<This module is fully compatible with ExtJS 3.0.>
 
 Check out the examples in C<examples/html>
 
-=for html <p> or online at [ <a href="http://search.cpan.org/src/PERLER/HTML-FormFu-ExtJS-0.07/examples/html">Examples</a> ]<p>
+=for html <p> or online at [ <a href="http://search.cpan.org/src/PERLER/HTML-FormFu-ExtJS-0.071/examples/html">Examples</a> ]<p>
 
 
 =head1 METHODS
@@ -197,12 +205,29 @@ sub _render {
 This method returns all form elements in the JavaScript Object Notation (JSON). You can put this string
 right into the C<items> attribute of your ExtJS form panel.
 
+Fields with the C<omit_rendering> attribute set to a true value are not rendered, they are ignored.
+This feature is usefull if you have fields that are autogenerated on the client side.
+
 =head2 _render_items
 
 Acts like L</render_items> but returns a perl object instead.
 
 =cut
 
+=head2 _render_item
+
+Renders a single element.
+
+=cut
+
+sub _render_item {
+    my $self = shift;
+    my $element = shift || $self;
+
+    my $class = ext_class_of( $element );
+    return $class->render($element);
+}
+
 sub _render_items {
 	my $self   = shift;
 	my $from   = shift || $self;
@@ -213,10 +238,17 @@ sub _render_items {
 			|| $element->type eq "Button"
 			|| $element->type eq "Reset"
 			|| $element->type eq "ContentButton" );
-		my $class = "HTML::FormFu::ExtJS::Element::" . $element->type;
-		require_class($class);
-		push( @{$output}, $class->render($element) );
+
+        # omit rendering field with 'omit_rendering' flag
+        next
+            if( defined $element->attributes()->{omit_rendering} && $element->attributes()->{omit_rendering} );
+
+        push( @{$output}, $self->_render_item( $element ) );
 	}
+
+    # remove undef elements
+    $output = [ grep { defined $_ } @{$output} ];
+
 	return $output;
 }
 
@@ -250,8 +282,7 @@ sub _render_buttons {
 			|| $element->type eq "Button"
 			|| $element->type eq "ContentButton"
 			|| $element->type eq "Reset" );
-		my $class = "HTML::FormFu::ExtJS::Element::" . $element->type;
-		require_class($class);
+        my $class = ext_class_of( $element );
 		push( @{$output}, $class->render($element) );
 	}
 	return $output;
@@ -347,11 +378,13 @@ sub _get_attributes {
 	my $obj = {};
 	foreach my $attr ( "attrs_xml", "attrs" ) {
 		my @keys = keys %{ $source->$attr };
-		for (@keys) {
-			$obj->{$_} =
-			  ref( $source->$attr->{$_} ) eq "HTML::FormFu::Literal"
-			  ? "" . $source->$attr->{$_}
-			  : $source->$attr->{$_};
+		for my $key (@keys) {
+            my $type = ref( $source->$attr->{$key} );
+            if ($type eq "HTML::FormFu::Literal") {
+                $obj->{$key} = "" . $source->$attr->{$key};
+            } else {
+                $obj->{$key} = $source->$attr->{$key};
+            }
 		}
 	}
 	return %{$obj};
@@ -404,13 +437,12 @@ sub validation_response {
 	$form->model('HashRef')->options(1);
 	$form->set_options(shift);
 	if ( $form->submitted_and_valid ) {
-		my $return = { success => 1 };
+		my $return = { success => \1 };
 		$form->default_values($form->params);
 		$return->{data} = $form->model('HashRef')->create;
 		return $return;
 	} elsif ( $form->has_errors ) {
-		my $return;
-		$return->{success} = 0;
+		my $return = { success => \0 };
 		for ( @{ $form->get_errors } ) {
 			push(
 				@{ $return->{errors} },
@@ -433,11 +465,25 @@ sub set_options {
 sub grid_data {
     my $self   = shift;
 	my $data = shift;
-	$self->model('HashRef')->flatten(0);
-	$self->model('HashRef')->options(1);
 	my $param  = shift;
-	$self->set_options(shift);
-    my $rows   = $self->ext_grid_data($data);
+	
+	croak 'First parameter has to be an array ref' unless(ref $data eq "ARRAY");
+	
+	my $rows = [];
+	
+	
+	if(blessed $data->[0]) {
+	
+    	$self->model('HashRef')->flatten(0);
+    	$self->model('HashRef')->options(1);
+    	$self->set_options(shift);
+        $rows   = $self->ext_grid_data($data);
+    
+    } elsif(ref $data->[0] eq "HASH") {
+        $rows   = $self->hashref_grid_data($data);
+    } elsif($data->[0]) {
+        croak 'Elements have to be a hash ref or blessed objects' . (ref $data->[0]);
+    }
     
     my $return = {
         results  => scalar @{$rows},
@@ -451,6 +497,70 @@ sub grid_data {
     return merge $return, $param;
 }
 
+
+sub hashref_grid_data {
+    #carp 'Mia sann im Hashref';
+    my $self = shift;
+    my $data = shift;
+    my @return;
+    my @all_elements = @{ $self->get_all_elements() };
+    my ( %element_cache, %deflator_cache, %inflator_cache, %options_cache );
+    foreach my $datum ( @{$data} ) {
+        my $obj;
+        foreach my $column (@all_elements) {
+            next if ( $column->type =~ /submit/i );
+            my $name    = $column->name;
+            next unless($name);
+            my $element = $element_cache{$name}
+              || $self->get_all_element($name);
+            $element_cache{$name} ||= $element;
+            next unless ($element);
+            $obj->{$name} = $datum->{$name};
+
+            my $inflators = $inflator_cache{$name}
+              || $element->get_inflators;
+            $inflator_cache{$name} ||= $inflators;
+
+            foreach my $inflator ( @{$inflators} ) {
+                $obj->{$name} = $inflator->inflator( $obj->{$name} );
+            }
+
+            my $deflators = $deflator_cache{$name}
+              || $element->get_deflators;
+            $deflator_cache{$name} ||= $deflators;
+
+            foreach my $deflator ( @{$deflators} ) {
+                $obj->{$name} = $deflator->deflator( $obj->{$name} );
+            }
+
+            if(ref $obj->{$name} || blessed $obj->{$name}) {
+                carp "$name was set to undef because it was either blessed or a reference";
+                $obj->{$name} = undef;
+                next;
+            }
+
+            my $can_options = $options_cache{$name}
+              || $element->can("_options");
+            $options_cache{$name} ||= $element->can("_options");
+            if ($can_options) {
+                my @options = @{ $element->_options };
+                my @option = grep { $_->{value} eq $obj->{$name} } @options;
+                unless(@option) {
+                    @options = map { @{$_->{group} || []} } @options;
+                    @option = grep { $_->{value} eq $obj->{$name} } @options ;
+                }
+                $obj->{$name} = { label => join(", ", map { $_->{label} } @option), value => $obj->{$name}};
+            }
+
+            #if($column->type eq "Checkbox") {
+            #    $obj->{$name} = \1 if($obj->{$name});
+            #}
+        }
+        push( @return, $obj );
+    }
+    return \@return;
+}
+
 sub form_data {
 	my $self = shift;
 	my $data = shift;
@@ -516,7 +626,7 @@ The first column is hidden and contains the value of the select box (e. g. C<0>
 second column contains the label of the value (e. g. C<male> or C<female>) and is visible.
 
 This way you can access both the value and the label of such a field. Notice the values of
-C<dataIndex> and <id> on those columns. Those correspond with the output of L</grid_data>.
+C<dataIndex> and C<id> on those columns. Those correspond with the output of L</grid_data>.
 
 =cut
 
@@ -529,8 +639,7 @@ sub _column_model {
     my @add  = @_;
     my $data;
     for my $element ( @{ $form->ext_columns() } ) {
-        my $class = "HTML::FormFu::ExtJS::Element::" . $element->type;
-        require_class($class);
+        my $class = ext_class_of( $element );
         push( @{$data}, $class->column_model($element) ) if ( $class->can("record") );
     }
 
@@ -549,8 +658,7 @@ sub _record {
     my @add  = @_;
     my $data;
     for my $element ( @{ $form->ext_columns() } ) {
-        my $class = "HTML::FormFu::ExtJS::Element::" . $element->type;
-        require_class($class);
+        my $class = ext_class_of( $element );
         push( @{$data}, $class->record($element) ) if ( $class->can("record") );
     }
 
@@ -560,62 +668,7 @@ sub _record {
     return $data;
 }
 
-# sub ext_grid_data {
-#     my $self = shift;
-#     my $data = shift;
-#     if ( blessed $data && $data->isa("DBIx::Class::ResultSet") ) {
-#         my @data = $data->all;
-#         $data = \@data;
-#     }
-#     my @return;
-#     my @all_elements = @{ $self->get_all_elements() };
-#     my ( %element_cache, %deflator_cache, %options_cache );
-#     foreach my $datum ( @{$data} ) {
-#         my $obj;
-#         foreach my $column (@all_elements) {
-#             next if ( $column->type =~ /submit/i );
-#             my $name    = $column->name;
-# 			next unless($name);
-#             my $element = $element_cache{$name}
-#               || $self->get_all_element($name);
-#             $element_cache{$name} ||= $element;
-#             next unless ($element);
-#             $obj->{$name} = blessed $datum && $datum->can($name) ? $datum->$name : $datum->{$name};
-# 			my $deflators = $deflator_cache{$name}
-#               || $element->get_deflators;
-#             $deflator_cache{$name} ||= $deflators;
-# 
-#             foreach my $deflator ( @{$deflators} ) {
-# 
-#                 $obj->{$name} = $deflator->deflator( $obj->{$name} );
-#             }
-# 
-# 			if(blessed $datum && $datum->can($name) && blessed $datum->$name && $datum->$name->can('count')) {
-# 				$obj->{$name} = $datum->$name->count;
-# 				next;
-# 			}
-# 
-#             my $can_options = $options_cache{$name}
-#               || $element->can("_options");
-#             $options_cache{$name} ||= $element->can("_options");
-#             if ($can_options) {
-#                 my @options = @{ $element->_options };
-#                 my @option = grep { $_->{value} eq $obj->{$name} } @options;
-# 				unless(@option) {
-# 					@options = map { @{$_->{group} || []} } @options;
-# 					@option = grep { $_->{value} eq $obj->{$name} } @options ;
-# 				}
-#                 $obj->{$name} = join(", ", map { $_->{label} } @option);
-#             }
-# 
-# 			if($column->type eq "Checkbox") {
-# 				$obj->{$name} = \1 if($obj->{$name});
-# 			}
-#         }
-#         push( @return, $obj );
-#     }
-#     return \@return;
-# }
+
 
 1;
 
@@ -664,6 +717,20 @@ so that you can access it from the template.
       $c->stash->{form} = $form;
       $c->stash->{template} = 'javascript.tt2';
   }
+  
+  sub create : Path('/contacts/create') {
+      my ($self, $c) = @_;
+      my $form = new HTML::FormFu::ExtJS;
+      $form->load_config_file('root/forms/form.yml');
+      $form->process($c->req->params);
+      if($form->submitted_and_valid) {
+          # insert into database
+      }
+      
+      $c->stash($form->validation_response);
+      # Make sure a JSON view is called so the stash is serialized
+  }
+  
 
 javascript.tt2:
 
@@ -671,13 +738,74 @@ javascript.tt2:
     form.getForm().submit({
 	  success: function(rst, req) {
         // submission was successful and valid
-    }
+        // you might want to close the form or something
+      },
+      failure: function() {
+        // form validation returned errors
+        // invalid fields are masked automatically  
+      }
   }
   var form = [% form.render %];
   
 
+
+=head1 FAQ
+
+=head2 How do I do server side validation?
+
+See L</simple form submission and validation> and L</validation_response>.
+
+=head2 And how do I add client side validation?
+
+The FormFu constraints are not yet translated to ExtJS constraints. You can however
+add them manually in your form config:
+
+  - type: Text
+    name: text
+    constraint:
+      - Required
+    attrs:
+      allowBlank: 0
+
+=head2 How do I create custom FormFu::ExtJS elements?
+
+If you wish to write your own ExtJS element you have to do the following:
+
+First create an element which is a HTML::FormFu::Element.
+
+  package HTML::FormFu::Element::MyApp::MyField;
+  use base qw(HTML::FormFu::Element::Text);
+  1;
+  
+This is a very basic example for a field which is a text field.
+The ExtJS logic belongs to a different module:
+
+  package HTML::FormFu::ExtJS::Element::MyApp::MyField;
+  use base qw(HTML::FormFu::ExtJS::Element::Text);
+  
+  sub render {
+      # you probably want to overwrite this!
+  }
+  
+  1;
+
+Configure the form as follows:
+
+  $form->populate( { elements => { type => 'MyApp::MyField', ... } } );
+  
+  
+If you don't want to put the element in the HTML::FormFu namespace then you have to
+prepend the class name with a C<+>. In this case the name of the ExtJS class changes
+as well:
+
+  $form->populate( { elements => { type => '+MyApp::Element::MyField', ... } } );
+
+This requires the classes C<MyApp::Element::MyField> and C<MyApp::ExtJS::Element::MyField>.
+The class must be in an C<Element> namespace.
+
 =head1 CAVEATS
 
+
 =head2 L<Multi|HTML::FormFu::ExtJS::Element::Multi>
 
 The Multi element is rendered using the ExtJS column layout. It seems like this 
@@ -690,13 +818,6 @@ where this hack failed. But there might be some cases where it does.
 Optgroups are partially supported. They render as a normal element of the select box and 
 are therefore selectable.
 
-=head2 L<File|HTML::FormFu::ExtJS::Element::File>
-
-With ExtJS 2.2 comes an option of the form panel which allows file uploads. Make sure you set
-C<fileUpload> at the form panel to C<true>.
-
-See L<http://extjs.com/deploy/dev/docs/?class=Ext.form.BasicForm> / fileUpload.
-
 =head2 L<Buttons|HTML::FormFu::ExtJS::Element::Button>
 
 Buttons cannot be placed in-line as ExtJS expects them to be in a different attribute. A
@@ -742,4 +863,4 @@ This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
 
 
-=cut
+=cut
\ No newline at end of file
